// 004 u_c_command_14
// 17 oct 2004

// -- Delphi implementation from the book:
// --   Design Patterns Elements of Reusable Object-Oriented Software
// --   Addison Wesley - 1994 - ISBN 0-201-63361-2
// -- (C) Erich GAMMA, Richard HELM, Ralph JOHNSON, John VLISSIDES 1994

// -- translation from C to Delphi performed by
// --   John COLIBRI  http://www.jcolibri.com

(*$r+*)

unit u_c_command_14;
  interface
    uses Classes;

    type // -- abstract
         c_command= class
                      procedure execute; Virtual; Abstract;
                    end; // c_command

         c_document= Class
                       constructor create_document(p_file_name: String);
                       procedure open;
                       procedure paste;
                     end; // c_document

         c_application= Class
                          constructor create_application;
                          procedure add_document(p_c_document: c_document);
                        end; // c_application

         c_open_command= class(c_command)
                           _m_c_application: c_application;
                           // _m_response: Char;

                           Constructor _create_open_command(p_c_application: c_application);
                           procedure execute; Override;
                           function f_ask_user: String; Virtual;
                           Destructor Destroy; Override;
                         end; // c_open_command

         c_paste_command= class(c_command)
                            _m_c_document: c_document;

                            Constructor _create_paste_command(p_c_document: c_document);
                            procedure execute; Override;
                            Destructor Destroy; Override;
                          end; // c_paste_command

         c_element= Class
                      constructor create_element;
                      procedure handle;
                    end; // c_element
         t_po_handle_element= Procedure of Object;

         c_simple_command= class(c_command)
                             _m_c_element: c_element;
                             _m_po_handle_element: t_po_handle_element;

                             constructor create_simple_command(p_c_element: c_element;
                                 p_po_handle_element: t_po_handle_element);
                             procedure execute; Override;
                             Destructor Destroy; Override;
                           end; // c_simple_command

         c_macro_command= Class(c_command)
                            _m_c_command_list: tList;

                            constructor create_macro_command;
                            procedure add_command(p_c_command: c_command); Virtual;
                            procedure remove_command(p_c_command: c_command); Virtual;
                            procedure execute; Override;
                            Destructor Destroy; Override;
                          end; // c_macro_command

  implementation

    // -- c_document

    constructor c_document.create_document(p_file_name: String);
      begin
        Inherited Create;
      end; // create_document

    procedure c_document.open;
      begin
      end; // open

    procedure c_document.paste;
      begin
      end; // paste

    // -- c_application

    constructor c_application.create_application;
      begin
        Inherited Create;
      end; // create_application

    procedure c_application.add_document(p_c_document: c_document);
      begin
      end; // add_document

    // -- c_open_command

    Constructor c_open_command._create_open_command(p_c_application: c_application);
      begin
        Inherited Create;
        _m_c_application:= p_c_application;
      end; // _create_open_command

    procedure c_open_command.execute;
      var l_file_name: String;
          l_c_document: c_document;
      begin
        l_file_name:= f_ask_user;
        if l_file_name<> ''
          then begin
              l_c_document:= c_document.create_document(l_file_name);
              _m_c_application.add_document(l_c_document);
            end;
      end; // execute

    function c_open_command.f_ask_user: String;
      begin
      end; // f_ask_user

    Destructor c_open_command.Destroy;
      begin
      end; // Destroy

    // -- c_paste_command

    Constructor c_paste_command._create_paste_command(p_c_document: c_document);
      begin
        Inherited Create;
        _m_c_document:= p_c_document;
      end; // _create_paste_command

    procedure c_paste_command.execute;
      begin
        _m_c_document.paste;
      end; // execute

    Destructor c_paste_command.Destroy;
      begin
      end; // Destroy

    // -- c_element

    constructor c_element.create_element;
      begin
      end; // create_element

    procedure c_element.handle;
      begin
      end; // handle

    // -- c_simple_command

    constructor c_simple_command.create_simple_command(p_c_element: c_element;
        p_po_handle_element: t_po_handle_element);
      begin
        Inherited Create;

        _m_c_element:= p_c_element;
        _m_po_handle_element:= p_po_handle_element;
      end; // create_simple_command

    procedure c_simple_command.execute;
      begin
        _m_po_handle_element;
      end; // execute

    Destructor c_simple_command.Destroy;
      begin
      end; // Destroy

    // -- c_macro_command

    constructor c_macro_command.create_macro_command;
      begin
      end; // create_macro_command

    procedure c_macro_command.add_command(p_c_command: c_command);
      begin
      end; // add_command

    procedure c_macro_command.remove_command(p_c_command: c_command);
      begin
      end; // remove_command

    procedure c_macro_command.execute;
      var l_command_index: Integer;
      begin
        for l_command_index:= 0 to _m_c_command_list.Count- 1 do
          c_command(_m_c_command_list[l_command_index]).execute;

      end; // execute

    Destructor c_macro_command.Destroy;
      begin
      end; // Destroy

    begin // u_c_command
    end. // u_c_command
